@@ -16,6 +16,7 @@ use function array_pop;
 use function array_slice;
 use function count;
 use function is_array;
+use function is_int;
 use function random_int;
 use function spl_object_id;
 use SplObjectStorage;
@@ -57,9 +58,10 @@ final class Context
      *
      * @param-out T $value
      */
-    public function add(array|object &$value): false|int|string
+    public function add(array|object &$value): int
     {
         if (is_array($value)) {
+            /* @phpstan-ignore paramOut.type */
             return $this->addArray($value);
         }
 
@@ -73,7 +75,7 @@ final class Context
      *
      * @param-out T $value
      */
-    public function contains(array|object &$value): false|int|string
+    public function contains(array|object &$value): false|int
     {
         if (is_array($value)) {
             return $this->containsArray($value);
@@ -126,8 +128,8 @@ final class Context
 
     private function addObject(object $object): int
     {
-        if (!$this->objects->contains($object)) {
-            $this->objects->attach($object);
+        if (!$this->objects->offsetExists($object)) {
+            $this->objects->offsetSet($object);
         }
 
         return spl_object_id($object);
@@ -140,12 +142,19 @@ final class Context
     {
         $end = array_slice($array, -2);
 
-        return isset($end[1]) && $end[1] === $this->objects ? $end[0] : false;
+        if (isset($end[1]) &&
+            $end[1] === $this->objects &&
+            isset($end[0]) &&
+            is_int($end[0])) {
+            return $end[0];
+        }
+
+        return false;
     }
 
     private function containsObject(object $value): false|int
     {
-        if ($this->objects->contains($value)) {
+        if ($this->objects->offsetExists($value)) {
             return spl_object_id($value);
         }
 
